@@ -4,7 +4,7 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -52,6 +52,7 @@ static gboolean enable_flag;
 static gboolean disable_flag;
 static gboolean set_power_state_on_flag;
 static gboolean set_power_state_low_flag;
+static gboolean set_power_state_off_flag;
 static gboolean reset_flag;
 static gchar *factory_reset_str;
 static gchar *command_str;
@@ -84,6 +85,10 @@ static GOptionEntry entries[] = {
       "Set low power state in the modem",
       NULL
     },
+    { "set-power-state-off", 0, 0, G_OPTION_ARG_NONE, &set_power_state_off_flag,
+      "Power off the modem",
+      NULL
+    },
     { "reset", 'r', 0, G_OPTION_ARG_NONE, &reset_flag,
       "Reset a given modem",
       NULL
@@ -130,17 +135,17 @@ static GOptionEntry entries[] = {
 GOptionGroup *
 mmcli_modem_get_option_group (void)
 {
-	GOptionGroup *group;
+    GOptionGroup *group;
 
-	/* Status options */
-	group = g_option_group_new ("modem",
-	                            "Modem options",
-	                            "Show modem options",
-	                            NULL,
-	                            NULL);
-	g_option_group_add_entries (group, entries);
+    /* Status options */
+    group = g_option_group_new ("modem",
+                                "Modem options",
+                                "Show modem options",
+                                NULL,
+                                NULL);
+    g_option_group_add_entries (group, entries);
 
-	return group;
+    return group;
 }
 
 gboolean
@@ -157,6 +162,7 @@ mmcli_modem_options_enabled (void)
                  disable_flag +
                  set_power_state_on_flag +
                  set_power_state_low_flag +
+                 set_power_state_off_flag +
                  reset_flag +
                  list_bearers_flag +
                  !!create_bearer_str +
@@ -244,6 +250,7 @@ print_modem_info (void)
 {
     gchar *drivers_string;
     gchar *prefixed_revision;
+    gchar *prefixed_hardware_revision;
     gchar *supported_capabilities_string;
     MMModemCapability *capabilities = NULL;
     guint n_capabilities = 0;
@@ -334,6 +341,12 @@ print_modem_info (void)
     else
         prefixed_revision = NULL;
 
+    if (mm_modem_get_hardware_revision (ctx->modem))
+        prefixed_hardware_revision = mmcli_prefix_newlines ("           |                  ",
+                                                            mm_modem_get_hardware_revision (ctx->modem));
+    else
+        prefixed_hardware_revision = NULL;
+
     if (supported_modes_string) {
         gchar *prefixed;
 
@@ -375,12 +388,14 @@ print_modem_info (void)
              "  Hardware |   manufacturer: '%s'\n"
              "           |          model: '%s'\n"
              "           |       revision: '%s'\n"
+             "           |   H/W revision: '%s'\n"
              "           |      supported: '%s'\n"
              "           |        current: '%s'\n"
              "           |   equipment id: '%s'\n",
              VALIDATE_UNKNOWN (mm_modem_get_manufacturer (ctx->modem)),
              VALIDATE_UNKNOWN (mm_modem_get_model (ctx->modem)),
              VALIDATE_UNKNOWN (prefixed_revision),
+             VALIDATE_UNKNOWN (prefixed_hardware_revision),
              VALIDATE_UNKNOWN (supported_capabilities_string),
              VALIDATE_UNKNOWN (current_capabilities_string),
              VALIDATE_UNKNOWN (mm_modem_get_equipment_identifier (ctx->modem)));
@@ -455,15 +470,18 @@ print_modem_info (void)
                  "           |    operator id: '%s'\n"
                  "           |  operator name: '%s'\n"
                  "           |   subscription: '%s'\n"
-                 "           |   registration: '%s'\n",
+                 "           |   registration: '%s'\n"
+                 "           |    EPS UE mode: '%s'\n",
                  VALIDATE_UNKNOWN (mm_modem_3gpp_get_imei (ctx->modem_3gpp)),
                  facility_locks,
                  VALIDATE_UNKNOWN (mm_modem_3gpp_get_operator_code (ctx->modem_3gpp)),
                  VALIDATE_UNKNOWN (mm_modem_3gpp_get_operator_name (ctx->modem_3gpp)),
                  mm_modem_3gpp_subscription_state_get_string (
-                     mm_modem_3gpp_get_subscription_state ((ctx->modem_3gpp))),
+                     mm_modem_3gpp_get_subscription_state (ctx->modem_3gpp)),
                  mm_modem_3gpp_registration_state_get_string (
-                     mm_modem_3gpp_get_registration_state ((ctx->modem_3gpp))));
+                     mm_modem_3gpp_get_registration_state (ctx->modem_3gpp)),
+                 mm_modem_3gpp_eps_ue_mode_operation_get_string (
+                     mm_modem_3gpp_get_eps_ue_mode_operation (ctx->modem_3gpp)));
 
         g_free (facility_locks);
     }
@@ -527,6 +545,7 @@ print_modem_info (void)
     g_free (supported_capabilities_string);
     g_free (current_capabilities_string);
     g_free (prefixed_revision);
+    g_free (prefixed_hardware_revision);
     g_free (allowed_modes_string);
     g_free (preferred_mode_string);
     g_free (supported_modes_string);
@@ -1071,6 +1090,17 @@ get_modem_ready (GObject      *source,
         return;
     }
 
+    /* Request to power off the modem? */
+    if (set_power_state_off_flag) {
+        g_debug ("Asynchronously powering off...");
+        mm_modem_set_power_state (ctx->modem,
+                                  MM_MODEM_POWER_STATE_OFF,
+                                  ctx->cancellable,
+                                  (GAsyncReadyCallback)set_power_state_ready,
+                                  NULL);
+        return;
+    }
+
     /* Request to reset the modem? */
     if (reset_flag) {
         g_debug ("Asynchronously reseting modem...");
@@ -1288,6 +1318,16 @@ mmcli_modem_run_synchronous (GDBusConnection *connection)
         return;
     }
 
+    /* Request to power off? */
+    if (set_power_state_off_flag) {
+        gboolean result;
+
+        g_debug ("Synchronously powering off...");
+        result = mm_modem_set_power_state_sync (ctx->modem, MM_MODEM_POWER_STATE_OFF, NULL, &error);
+        set_power_state_process_reply (result, error);
+        return;
+    }
+
     /* Request to reset the modem? */
     if (reset_flag) {
         gboolean result;
