@@ -20,7 +20,7 @@
 #include <libmm-glib.h>
 
 #include "mm-bearer-list.h"
-#include "mm-sim.h"
+#include "mm-base-sim.h"
 #include "mm-error-helpers.h"
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-3gpp.h"
@@ -32,7 +32,6 @@
 /* Register in either a CDMA or a 3GPP network (or both) */
 
 typedef struct {
-    GSimpleAsyncResult *result;
     MMIfaceModemSimple *self;
     gchar *operator_id;
     guint remaining_tries_cdma;
@@ -41,11 +40,9 @@ typedef struct {
 } RegisterInNetworkContext;
 
 static void
-register_in_network_context_complete_and_free (RegisterInNetworkContext *ctx)
+register_in_network_context_free (RegisterInNetworkContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
     g_free (ctx->operator_id);
-    g_object_unref (ctx->result);
     g_object_unref (ctx->self);
     g_free (ctx);
 }
@@ -55,59 +52,69 @@ register_in_3gpp_or_cdma_network_finish (MMIfaceModemSimple *self,
                                          GAsyncResult *res,
                                          GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static void check_next_registration (RegisterInNetworkContext *ctx);
+static void check_next_registration (GTask *task);
 
 static void
 register_in_cdma_network_ready (MMIfaceModemCdma *self,
                                 GAsyncResult *res,
-                                RegisterInNetworkContext *ctx)
+                                GTask *task)
 {
+    RegisterInNetworkContext *ctx;
+
+    ctx = g_task_get_task_data (task);
     ctx->remaining_tries_cdma--;
 
     if (!mm_iface_modem_cdma_register_in_network_finish (
             MM_IFACE_MODEM_CDMA (self), res, NULL)) {
         /* Retry check */
-        check_next_registration (ctx);
+        check_next_registration (task);
         return;
     }
 
     /* Registered we are! */
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    register_in_network_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 static void
 register_in_3gpp_network_ready (MMIfaceModem3gpp *self,
                                 GAsyncResult *res,
-                                RegisterInNetworkContext *ctx)
+                                GTask *task)
 {
+    RegisterInNetworkContext *ctx;
+
+    ctx = g_task_get_task_data (task);
     ctx->remaining_tries_3gpp--;
 
     if (!mm_iface_modem_3gpp_register_in_network_finish (
             MM_IFACE_MODEM_3GPP (self), res, NULL)) {
         /* Retry check */
-        check_next_registration (ctx);
+        check_next_registration (task);
         return;
     }
 
     /* Registered we are! */
-    g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    register_in_network_context_complete_and_free (ctx);
+    g_task_return_boolean (task, TRUE);
+    g_object_unref (task);
 }
 
 static void
-check_next_registration (RegisterInNetworkContext *ctx)
+check_next_registration (GTask *task)
 {
+    RegisterInNetworkContext *ctx;
+
+    ctx = g_task_get_task_data (task);
+
     if (ctx->remaining_tries_cdma > ctx->remaining_tries_3gpp &&
         ctx->remaining_tries_cdma > 0) {
         mm_iface_modem_cdma_register_in_network (
             MM_IFACE_MODEM_CDMA (ctx->self),
             ctx->max_try_time,
             (GAsyncReadyCallback)register_in_cdma_network_ready,
-            ctx);
+            task);
         return;
     }
 
@@ -117,15 +124,15 @@ check_next_registration (RegisterInNetworkContext *ctx)
             ctx->operator_id,
             ctx->max_try_time,
             (GAsyncReadyCallback)register_in_3gpp_network_ready,
-            ctx);
+            task);
         return;
     }
 
     /* No more tries of anything */
-    g_simple_async_result_take_error (
-        ctx->result,
+    g_task_return_error (
+        task,
         mm_mobile_equipment_error_for_code (MM_MOBILE_EQUIPMENT_ERROR_NETWORK_TIMEOUT));
-    register_in_network_context_complete_and_free (ctx);
+    g_object_unref (task);
 }
 
 static void
@@ -135,14 +142,11 @@ register_in_3gpp_or_cdma_network (MMIfaceModemSimple *self,
                                   gpointer user_data)
 {
     RegisterInNetworkContext *ctx;
+    GTask *task;
 
     ctx = g_new0 (RegisterInNetworkContext, 1);
     ctx->self = g_object_ref (self);
     ctx->operator_id = g_strdup (operator_id);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             register_in_3gpp_or_cdma_network);
 
     /* 3GPP-only modems... */
     if (mm_iface_modem_is_3gpp_only (MM_IFACE_MODEM (ctx->self))) {
@@ -163,7 +167,12 @@ register_in_3gpp_or_cdma_network (MMIfaceModemSimple *self,
         ctx->remaining_tries_3gpp = 6;
     }
 
-    check_next_registration (ctx);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task,
+                          ctx,
+                          (GDestroyNotify)register_in_network_context_free);
+
+    check_next_registration (task);
 }
 
 /*****************************************************************************/
@@ -174,8 +183,6 @@ typedef enum {
     CONNECTION_STEP_WAIT_FOR_INITIALIZED,
     CONNECTION_STEP_ENABLE,
     CONNECTION_STEP_WAIT_FOR_ENABLED,
-    CONNECTION_STEP_ALLOWED_MODES,
-    CONNECTION_STEP_BANDS,
     CONNECTION_STEP_REGISTER,
     CONNECTION_STEP_BEARER,
     CONNECTION_STEP_CONNECT,
@@ -187,23 +194,18 @@ typedef struct {
     GDBusMethodInvocation *invocation;
     MMIfaceModemSimple *self;
     ConnectionStep step;
-    gulong state_changed_id;
-    guint state_changed_wait_id;
 
     /* Expected input properties */
     GVariant *dictionary;
     MMSimpleConnectProperties *properties;
 
     /* Results to set */
-    MMBearer *bearer;
+    MMBaseBearer *bearer;
 } ConnectionContext;
 
 static void
 connection_context_free (ConnectionContext *ctx)
 {
-    g_assert (ctx->state_changed_id == 0);
-    g_assert (ctx->state_changed_wait_id == 0);
-
     g_variant_unref (ctx->dictionary);
     if (ctx->properties)
         g_object_unref (ctx->properties);
@@ -218,13 +220,13 @@ connection_context_free (ConnectionContext *ctx)
 static void connection_step (ConnectionContext *ctx);
 
 static void
-connect_bearer_ready (MMBearer *bearer,
+connect_bearer_ready (MMBaseBearer *bearer,
                       GAsyncResult *res,
                       ConnectionContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_bearer_connect_finish (bearer, res, &error)) {
+    if (!mm_base_bearer_connect_finish (bearer, res, &error)) {
         mm_dbg ("Couldn't connect bearer: '%s'", error->message);
         g_dbus_method_invocation_take_error (ctx->invocation, error);
         connection_context_free (ctx);
@@ -274,82 +276,6 @@ register_in_3gpp_or_cdma_network_ready (MMIfaceModemSimple *self,
     connection_step (ctx);
 }
 
-static gboolean
-after_set_allowed_modes_timeout_cb (ConnectionContext *ctx)
-{
-    /* Allowed modes set... almost there! */
-    ctx->step++;
-    connection_step (ctx);
-    return FALSE;
-}
-
-static void
-set_allowed_modes_ready (MMBaseModem *self,
-                         GAsyncResult *res,
-                         ConnectionContext *ctx)
-{
-    GError *error = NULL;
-
-    if (!mm_iface_modem_set_allowed_modes_finish (MM_IFACE_MODEM (self), res, &error)) {
-        if (g_error_matches (error,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_UNSUPPORTED)) {
-            g_error_free (error);
-            /* If setting bands is unsupported, keep on without sleep */
-            ctx->step++;
-            connection_step (ctx);
-        } else {
-            g_dbus_method_invocation_take_error (ctx->invocation, error);
-            connection_context_free (ctx);
-        }
-        return;
-    }
-
-    /* Setting allowed modes will reset the current registration, so we'll need
-     * a couple of seconds to settle down. This sleep time just makes sure that
-     * the modem has enough time to report being unregistered. */
-    mm_dbg ("Will wait to settle down after updating allowed modes");
-    g_timeout_add_seconds (2, (GSourceFunc)after_set_allowed_modes_timeout_cb, ctx);
-}
-
-static gboolean
-after_set_bands_timeout_cb (ConnectionContext *ctx)
-{
-    /* Bands set... almost there! */
-    ctx->step++;
-    connection_step (ctx);
-    return FALSE;
-}
-
-static void
-set_bands_ready (MMBaseModem *self,
-                 GAsyncResult *res,
-                 ConnectionContext *ctx)
-{
-    GError *error = NULL;
-
-    if (!mm_iface_modem_set_bands_finish (MM_IFACE_MODEM (self), res, &error)) {
-        if (g_error_matches (error,
-                             MM_CORE_ERROR,
-                             MM_CORE_ERROR_UNSUPPORTED)) {
-            g_error_free (error);
-            /* If setting bands is unsupported, keep on without sleep */
-            ctx->step++;
-            connection_step (ctx);
-        } else {
-            g_dbus_method_invocation_take_error (ctx->invocation, error);
-            connection_context_free (ctx);
-        }
-        return;
-    }
-
-    /* Setting bands will reset the current registration, so we'll need a couple
-     * of seconds to settle down. This sleep time just makes sure that the modem
-     * has enough time to report being unregistered. */
-    mm_dbg ("Will wait to settle down after updating bands");
-    g_timeout_add_seconds (2, (GSourceFunc)after_set_bands_timeout_cb, ctx);
-}
-
 static void
 wait_for_enabled_ready (MMIfaceModem *self,
                         GAsyncResult *res,
@@ -419,13 +345,13 @@ wait_for_initialized_ready (MMIfaceModem *self,
 }
 
 static void
-send_pin_ready (MMSim *sim,
+send_pin_ready (MMBaseSim *sim,
                 GAsyncResult *res,
                 ConnectionContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_sim_send_pin_finish (sim, res, &error)) {
+    if (!mm_base_sim_send_pin_finish (sim, res, &error)) {
         g_dbus_method_invocation_take_error (ctx->invocation, error);
         connection_context_free (ctx);
         return;
@@ -437,15 +363,15 @@ send_pin_ready (MMSim *sim,
 }
 
 static void
-unlock_check_ready (MMIfaceModem *self,
-                    GAsyncResult *res,
-                    ConnectionContext *ctx)
+update_lock_info_ready (MMIfaceModem *self,
+                        GAsyncResult *res,
+                        ConnectionContext *ctx)
 {
     GError *error = NULL;
     MMModemLock lock;
-    MMSim *sim;
+    MMBaseSim *sim;
 
-    lock = mm_iface_modem_unlock_check_finish (self, res, &error);
+    lock = mm_iface_modem_update_lock_info_finish (self, res, &error);
     if (error) {
         g_dbus_method_invocation_take_error (ctx->invocation, error);
         connection_context_free (ctx);
@@ -490,26 +416,26 @@ unlock_check_ready (MMIfaceModem *self,
         return;
     }
 
-    mm_sim_send_pin (sim,
-                     mm_simple_connect_properties_get_pin (ctx->properties),
-                     (GAsyncReadyCallback)send_pin_ready,
-                     ctx);
+    mm_base_sim_send_pin (sim,
+                          mm_simple_connect_properties_get_pin (ctx->properties),
+                          (GAsyncReadyCallback)send_pin_ready,
+                          ctx);
     g_object_unref (sim);
 }
 
 typedef struct {
-    MMBearer *found;
+    MMBaseBearer *found;
 } BearerListFindContext;
 
 static void
-bearer_list_find_disconnected (MMBearer *bearer,
+bearer_list_find_disconnected (MMBaseBearer *bearer,
                                BearerListFindContext *ctx)
 {
     /* If already marked one to remove, do nothing */
     if (ctx->found)
         return;
 
-    if (mm_bearer_get_status (bearer) == MM_BEARER_STATUS_DISCONNECTED)
+    if (mm_base_bearer_get_status (bearer) == MM_BEARER_STATUS_DISCONNECTED)
         ctx->found = g_object_ref (bearer);
 }
 
@@ -524,9 +450,10 @@ connection_step (ConnectionContext *ctx)
     case CONNECTION_STEP_UNLOCK_CHECK:
         mm_info ("Simple connect state (%d/%d): Unlock check",
                  ctx->step, CONNECTION_STEP_LAST);
-        mm_iface_modem_unlock_check (MM_IFACE_MODEM (ctx->self),
-                                     (GAsyncReadyCallback)unlock_check_ready,
-                                     ctx);
+        mm_iface_modem_update_lock_info (MM_IFACE_MODEM (ctx->self),
+                                         MM_MODEM_LOCK_UNKNOWN, /* ask */
+                                         (GAsyncReadyCallback)update_lock_info_ready,
+                                         ctx);
         return;
 
     case CONNECTION_STEP_WAIT_FOR_INITIALIZED:
@@ -555,61 +482,6 @@ connection_step (ConnectionContext *ctx)
                                              ctx);
         return;
 
-    case CONNECTION_STEP_ALLOWED_MODES: {
-        MMModemMode allowed_modes = MM_MODEM_MODE_ANY;
-        MMModemMode preferred_mode = MM_MODEM_MODE_NONE;
-
-        mm_info ("Simple connect state (%d/%d): Allowed mode",
-                 ctx->step, CONNECTION_STEP_LAST);
-
-        /* Don't set modes unless explicitly requested to do so */
-        if (mm_simple_connect_properties_get_allowed_modes (ctx->properties,
-                                                            &allowed_modes,
-                                                            &preferred_mode)) {
-            mm_iface_modem_set_allowed_modes (MM_IFACE_MODEM (ctx->self),
-                                              allowed_modes,
-                                              preferred_mode,
-                                              (GAsyncReadyCallback)set_allowed_modes_ready,
-                                              ctx);
-            return;
-        }
-
-        /* Fall down to next step */
-        ctx->step++;
-    }
-
-    case CONNECTION_STEP_BANDS: {
-        const MMModemBand *bands = NULL;
-        guint n_bands = 0;
-
-        mm_info ("Simple connect state (%d/%d): Bands",
-                 ctx->step, CONNECTION_STEP_LAST);
-
-        /* Don't set bands unless explicitly requested to do so */
-        if (mm_simple_connect_properties_get_bands (ctx->properties,
-                                                    &bands,
-                                                    &n_bands)) {
-            GArray *array;
-            guint i;
-
-            if (bands && *bands) {
-                array = g_array_sized_new (FALSE, FALSE, sizeof (MMModemBand), n_bands);
-                for (i = 0; i < n_bands; i++)
-                    g_array_insert_val (array, i, bands[i]);
-
-                mm_iface_modem_set_bands (MM_IFACE_MODEM (ctx->self),
-                                          array,
-                                          (GAsyncReadyCallback)set_bands_ready,
-                                          ctx);
-                g_array_unref (array);
-                return;
-            }
-        }
-
-        /* Fall down to next step */
-        ctx->step++;
-    }
-
     case CONNECTION_STEP_REGISTER:
         mm_info ("Simple connect state (%d/%d): Register",
                  ctx->step, CONNECTION_STEP_LAST);
@@ -653,7 +525,7 @@ connection_step (ConnectionContext *ctx)
         bearer_properties = mm_simple_connect_properties_get_bearer_properties (ctx->properties);
 
         /* Check if the bearer we want to create is already in the list */
-        ctx->bearer = mm_bearer_list_find (list, bearer_properties);
+        ctx->bearer = mm_bearer_list_find_by_properties (list, bearer_properties);
         if (!ctx->bearer) {
             mm_dbg ("Creating new bearer...");
             /* If we don't have enough space to create the bearer, try to remove
@@ -671,15 +543,15 @@ connection_step (ConnectionContext *ctx)
                     GError *error = NULL;
 
                     if (!mm_bearer_list_delete_bearer (list,
-                                                       mm_bearer_get_path (foreach_ctx.found),
+                                                       mm_base_bearer_get_path (foreach_ctx.found),
                                                        &error)) {
                         mm_dbg ("Couldn't delete disconnected bearer at '%s': '%s'",
-                                mm_bearer_get_path (foreach_ctx.found),
+                                mm_base_bearer_get_path (foreach_ctx.found),
                                 error->message);
                         g_error_free (error);
                     } else
                         mm_dbg ("Deleted disconnected bearer at '%s'",
-                                mm_bearer_get_path (foreach_ctx.found));
+                                mm_base_bearer_get_path (foreach_ctx.found));
                     g_object_unref (foreach_ctx.found);
                 }
 
@@ -708,7 +580,7 @@ connection_step (ConnectionContext *ctx)
         }
 
         mm_dbg ("Using already existing bearer at '%s'...",
-                mm_bearer_get_path (ctx->bearer));
+                mm_base_bearer_get_path (ctx->bearer));
         g_object_unref (list);
         g_object_unref (bearer_properties);
         /* Fall down to next step */
@@ -721,15 +593,15 @@ connection_step (ConnectionContext *ctx)
 
         /* Wait... if we're already using an existing bearer, we need to check if it is
          * already connected; and if so, just don't do anything else */
-        if (mm_bearer_get_status (ctx->bearer) != MM_BEARER_STATUS_CONNECTED) {
-            mm_bearer_connect (ctx->bearer,
-                               (GAsyncReadyCallback)connect_bearer_ready,
-                               ctx);
+        if (mm_base_bearer_get_status (ctx->bearer) != MM_BEARER_STATUS_CONNECTED) {
+            mm_base_bearer_connect (ctx->bearer,
+                                    (GAsyncReadyCallback)connect_bearer_ready,
+                                    ctx);
             return;
         }
 
         mm_dbg ("Bearer at '%s' is already connected...",
-                mm_bearer_get_path (ctx->bearer));
+                mm_base_bearer_get_path (ctx->bearer));
 
         /* Fall down to next step */
         ctx->step++;
@@ -741,7 +613,7 @@ connection_step (ConnectionContext *ctx)
         mm_gdbus_modem_simple_complete_connect (
             ctx->skeleton,
             ctx->invocation,
-            mm_bearer_get_path (ctx->bearer));
+            mm_base_bearer_get_path (ctx->bearer));
         connection_context_free (ctx);
         return;
     }
@@ -777,6 +649,47 @@ connect_auth_ready (MMBaseModem *self,
 
     mm_info ("Simple connect started...");
 
+    /* Log about all the parameters being used for the simple connect */
+    {
+        MMBearerAllowedAuth allowed_auth;
+        gchar *str;
+        MMBearerIpFamily ip_family;
+
+#define VALIDATE_UNSPECIFIED(str) (str ? str : "unspecified")
+
+        mm_dbg ("   PIN: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_pin (ctx->properties)));
+
+        mm_dbg ("   Operator ID: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_operator_id (ctx->properties)));
+
+        mm_dbg ("   Allowed roaming: %s", mm_simple_connect_properties_get_allow_roaming (ctx->properties) ? "yes" : "no");
+
+        mm_dbg ("   APN: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_apn (ctx->properties)));
+
+        ip_family = mm_simple_connect_properties_get_ip_type (ctx->properties);
+        if (ip_family != MM_BEARER_IP_FAMILY_NONE) {
+            str = mm_bearer_ip_family_build_string_from_mask (ip_family);
+            mm_dbg ("   IP family: %s", str);
+            g_free (str);
+        } else
+            mm_dbg ("   IP family: %s", VALIDATE_UNSPECIFIED (NULL));
+
+        allowed_auth = mm_simple_connect_properties_get_allowed_auth (ctx->properties);
+        if (allowed_auth != MM_BEARER_ALLOWED_AUTH_UNKNOWN) {
+            str = mm_bearer_allowed_auth_build_string_from_mask (allowed_auth);
+            mm_dbg ("   Allowed authentication: %s", str);
+            g_free (str);
+        } else
+            mm_dbg ("   Allowed authentication: %s", VALIDATE_UNSPECIFIED (NULL));
+
+        mm_dbg ("   User: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_user (ctx->properties)));
+
+        mm_dbg ("   Password: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_password (ctx->properties)));
+
+        mm_dbg ("   Number: %s", VALIDATE_UNSPECIFIED (mm_simple_connect_properties_get_number (ctx->properties)));
+
+#undef VALIDATE_UNSPECIFIED
+    }
+
     switch (current) {
     case MM_MODEM_STATE_FAILED:
     case MM_MODEM_STATE_UNKNOWN:
@@ -844,7 +757,7 @@ typedef struct {
     GDBusMethodInvocation *invocation;
     gchar *bearer_path;
     GList *bearers;
-    MMBearer *current;
+    MMBaseBearer *current;
 } DisconnectionContext;
 
 static void
@@ -856,20 +769,20 @@ disconnection_context_free (DisconnectionContext *ctx)
     g_free (ctx->bearer_path);
     if (ctx->current)
         g_object_unref (ctx->current);
-    g_list_free_full (ctx->bearers, (GDestroyNotify) g_object_unref);
+    g_list_free_full (ctx->bearers, g_object_unref);
     g_free (ctx);
 }
 
 static void disconnect_next_bearer (DisconnectionContext *ctx);
 
 static void
-disconnect_ready (MMBearer *bearer,
+disconnect_ready (MMBaseBearer *bearer,
                   GAsyncResult *res,
                   DisconnectionContext *ctx)
 {
     GError *error = NULL;
 
-    if (!mm_bearer_disconnect_finish (bearer, res, &error)) {
+    if (!mm_base_bearer_disconnect_finish (bearer, res, &error)) {
         g_dbus_method_invocation_take_error (ctx->invocation, error);
         disconnection_context_free (ctx);
         return;
@@ -892,20 +805,20 @@ disconnect_next_bearer (DisconnectionContext *ctx)
         return;
     }
 
-    ctx->current = MM_BEARER (ctx->bearers->data);
+    ctx->current = MM_BASE_BEARER (ctx->bearers->data);
     ctx->bearers = g_list_delete_link (ctx->bearers, ctx->bearers);
 
-    mm_bearer_disconnect (ctx->current,
-                          (GAsyncReadyCallback)disconnect_ready,
-                          ctx);
+    mm_base_bearer_disconnect (ctx->current,
+                               (GAsyncReadyCallback)disconnect_ready,
+                               ctx);
 }
 
 static void
-build_connected_bearer_list (MMBearer *bearer,
+build_connected_bearer_list (MMBaseBearer *bearer,
                              DisconnectionContext *ctx)
 {
     if (!ctx->bearer_path ||
-        g_str_equal (ctx->bearer_path, mm_bearer_get_path (bearer)))
+        g_str_equal (ctx->bearer_path, mm_base_bearer_get_path (bearer)))
         ctx->bearers = g_list_prepend (ctx->bearers, g_object_ref (bearer));
 }
 
